﻿--[[ Postal_Select: Adds checkboxes to the inbox with buttons for returning/opening. ]]--

assert( Postal, "Postal not found!")

------------------------------
--      Are you local?      --
------------------------------

local L = AceLibrary("AceLocale-2.2"):new("Postal")
local _G = getfenv(0)

local currentMode = nil
local toProcess = 1
local selectedMail = {}

local openButton = nil
local returnButton = nil

local checkboxFunc = function() Postal_Select:ToggleMail(this) end

----------------------------------
--      Module Declaration      --
----------------------------------

Postal_Select = Postal:NewModule("Select")
Postal_Select.revision = tonumber(string.sub("$Revision: 48356 $", 12, -3))

function Postal_Select:OnEnable()
	--create the open button
	if not openButton then
		openButton = CreateFrame("Button", "PostalSelectOpenButton", InboxFrame, "UIPanelButtonTemplate")
		openButton:SetWidth(120)
		openButton:SetHeight(25)
		openButton:SetPoint("RIGHT", InboxFrame, "TOP", 5, -53)
		openButton:SetText(L["Open"])
		openButton:SetScript("OnClick", function() Postal_Select:HandleSelect(1) end)
	end

	--create the return button
	if not returnButton then
		returnButton = CreateFrame("Button", "PostalSelectReturnButton", InboxFrame, "UIPanelButtonTemplate")
		returnButton:SetWidth(120)
		returnButton:SetHeight(25)
		returnButton:SetPoint("LEFT", InboxFrame, "TOP", 10, -53)
		returnButton:SetText(L["Return"])
		returnButton:SetScript("OnClick", function() Postal_Select:HandleSelect() end)
	end

	--indent to make room for the checkboxes
	MailItem1:SetPoint("TOPLEFT", "InboxFrame", "TOPLEFT", 48, -80)
	for i = 1, 7 do
		_G["MailItem"..i.."ExpireTime"]:SetPoint("TOPRIGHT", "MailItem"..i, "TOPRIGHT", 10, -4)
		_G["MailItem"..i]:SetWidth(280)
	end

	--now create the checkboxes
	for i = 1, 7 do
		if not _G["PostalInboxCB"..i] then
			local CB = CreateFrame("CheckButton", "PostalInboxCB"..i, _G["MailItem"..i], "OptionsCheckButtonTemplate")
			CB:SetID(i)
			CB:SetPoint("RIGHT", "MailItem"..i, "LEFT", 1, 0)
			CB:SetWidth(24)
			CB:SetHeight(24)
			CB:SetHitRectInsets(0, 0, 0, 0)
			CB:SetScript("OnClick", checkboxFunc)
		end
	end

	--hook shizz
	self:Hook("InboxFrame_Update", true)
	--make sure to reset on mail close
	self:RegisterEvent("MAIL_CLOSE", "Reset")
	self:RegisterEvent("PLAYER_LEAVING_WORLD", "Reset")
end

function Postal_Select:ToggleMail(frame)
	local index = frame:GetID() + (InboxFrame.pageNum - 1) * 7
	if not frame:GetChecked() then
		self.core:tremovebyval(selectedMail, index)
	else
		table.insert(selectedMail, index)
	end
end

function Postal_Select:HandleSelect(mode)
	if not selectedMail[1] then return end
	currentMode = mode
	if currentMode then
		openButton:SetText(L["In Progress"])
		returnButton:Hide()
	else
		returnButton:SetText(L["In Progress"])
		openButton:Hide()
	end

	--protect the user from changing anything while were in process
	self.core:DisableInbox(1)
	if self:IsHooked("InboxFrame_Update") then
		self:Unhook("InboxFrame_Update")
	end

	for i = 1, 7 do
		local index = i + (InboxFrame.pageNum-1)*7
		local CB = _G["PostalInboxCB"..i]
		CB:Hide()
	end

	toProcess = #selectedMail
	table.sort(selectedMail)

	self:RegisterEvent("Postal_Select_Next", "ProcessNext")
	toProcess = toProcess + 1
	self:ProcessNext()


end

function Postal_Select:ProcessNext()
	toProcess = toProcess - 1

	if toProcess > 0 then
		local msgSubject = select(4, GetInboxHeaderInfo(selectedMail[toProcess]))
		if currentMode then
			self.core:Print(L["Open"].." "..selectedMail[toProcess]..": "..msgSubject)
			self:SnarfMail()
		else
			self.core:Print(L["Return"].." "..selectedMail[toProcess]..": "..msgSubject)
			self:ReturnMail()
		end
	else
		self:Reset()
	end
end

function Postal_Select:InboxFrame_Update()
	self.hooks["InboxFrame_Update"]()
	for i = 1, 7 do
		local index = i + (InboxFrame.pageNum-1)*7
		local CB = _G["PostalInboxCB"..i]
		if index > GetInboxNumItems() then
			CB:Hide()
		else
			CB:Show()
			CB:SetChecked(nil)
			for key, value in pairs(selectedMail) do
				if value == index then
					CB:SetChecked(1)
					break
				end
			end
		end
	end
end

function Postal_Select:SnarfMail()
	local _, msgMoney, msgCOD, _, msgItem, _, _, _ = select(4, GetInboxHeaderInfo(selectedMail[toProcess]))

	if msgCOD and msgCOD > 0 then
		self.core:Print(L["Skipping"].." "..selectedMail[toProcess]..": "..L["COD"])
		self:TriggerEvent("Postal_Select_Next")
		return
	end

	if msgItem then
		if not self.core:SpaceCheck() then
			self.core:Print(L["Skipping"].." "..selectedMail[toProcess]..": "..L["Insufficient Bag Space"])
			self:TriggerEvent("Postal_Select_Next")
			return
		end
		-- self.core:Print("Getting Item from Message "..mailIndex)
		self:RegisterEvent("MAIL_INBOX_UPDATE")
		TakeInboxItem(selectedMail[toProcess])
	elseif msgMoney > 0 then
	--	self.core:Print("Looting Message "..mailIndex)
		self:RegisterEvent("MAIL_INBOX_UPDATE")
		TakeInboxMoney(selectedMail[toProcess])
	else
	--	self.core:Print("Deleting Message "..mailIndex)
		self:RegisterEvent("MAIL_INBOX_UPDATE")
		DeleteInboxItem(selectedMail[toProcess])
	end
end

function Postal_Select:MAIL_INBOX_UPDATE()
	--self.core:Print("update")
	self:UnregisterEvent("MAIL_INBOX_UPDATE")
	self:ScheduleEvent("Postal_Select_Next", .5)
end

function Postal_Select:ReturnMail()
	local wasReturned, _, canReply = select(10, GetInboxHeaderInfo(selectedMail[toProcess]))
	if not wasReturned and canReply then
		self:RegisterEvent("MAIL_INBOX_UPDATE")
		ReturnInboxItem(selectedMail[toProcess])
	else
		self.core:Print(L["Skipping"].." "..selectedMail[toProcess])
	end
end

function Postal_Select:Reset()
	if not self:IsHooked("InboxFrame_Update") then 	self:Hook("InboxFrame_Update", true) end

	if self:IsEventRegistered("Postal_Select_Next") then self:UnregisterEvent("Postal_Select_Next") end

	for k in pairs(selectedMail) do
		selectedMail[k] = nil
	end

	toProcess = 1
	self.core:DisableInbox()
	InboxFrame_Update()
	openButton:SetText(L["Open"])
	openButton:Show()
	returnButton:SetText(L["Return"])
	returnButton:Show()
end
